@@ -112,8 +112,18 @@ def plot_simulation_output(
     row_idx = 0
     col_idx = 0
     for port in port_lengths.keys():
-        # get this port's states
-        port_timeseries = {path[1:]: ts for path, ts in timeseries.items() if path[0] is port}
+
+        # get this port's timeseries
+        port_timeseries = {}
+        for path, ts in timeseries.items():
+            if path[0] is port:
+                next_path = path[1:]
+                if any(isinstance(item, tuple) for item in next_path):
+                    next_path = tuple([
+                        item[0] if isinstance(item, tuple) else item
+                        for item in next_path])
+                port_timeseries[next_path] = ts
+
         for state_id, series in sorted(port_timeseries.items()):
             if remove_first_timestep:
                 series = series[1:]
